/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.listener;

import java.util.logging.Level;

import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;
import javax.xml.transform.TransformerException;

import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.BrokeredNotificationService;
import org.ow2.petals.ServiceEngine;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.ow2.petals.component.framework.util.XMLUtil;
import org.w3c.dom.Document;

import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationReader;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationWriter;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.WsnbConstants;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.WsnbrConstants;
import com.ebmwebsourcing.wsstar.notification.service.fault.WSNotificationFault;
import com.ebmwebsourcing.wsstar.notification.service.util.WSNotificationNotImplementedException;

/**
 * @author Gardes Frederic
 * @author tdejean - eBM WebSourcing
 * 
 */
public class JBIListener extends AbstractJBIListener {

    private final QName GET_SUPPORTED_TOPICS_QNAME = new QName("http://petals.ow2.org/petals-se-notification","GetSupportedTopics");
    
    /**
     * Creates a new instance of {@link JBIListener}
     * 
     */
    public JBIListener() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.listener.AbstractJBIListener#onJBIMessage
     * (org.objectweb.petals.component.framework.util.Exchange)
     */
    @Override
    public boolean onJBIMessage(final Exchange exchange) {
        if (this.getLogger().isLoggable(Level.FINE)) {
            this.getLogger().log(Level.FINE,
                    "Notification Message received on onJBIMessage method !");
        }
        return this.processMessageExchange(exchange);

    }
    
    @Override
    public boolean onNotificationMessage(final Exchange exchange) {
        if (this.getLogger().isLoggable(Level.FINE)) {
            this.getLogger().log(Level.FINE,
                    "Notification Message received on 'onNotificationMessage' method !");
        }
        return this.processMessageExchange(exchange);
    }

    /**
     * Process message exchange for both standard JBI message and notification
     * 
     * @param exchange
     * @return
     */
    private boolean processMessageExchange(final Exchange exchange) {
        Document outWsnContent = null;
        Fault fault = null;

        final BrokeredNotificationService wsnbrService = ((ServiceEngine) this.getComponent()).getService();
        final WSNotificationReader wsnReader = ((ServiceEngine) this.getComponent()).getWsnReader();
        final WSNotificationWriter wsnWriter = ((ServiceEngine) this.getComponent()).getWsnWriter();

        try {
            
            final String opName = exchange.getOperation().getLocalPart();
            final Document docContent = exchange.getInMessageContentAsDocument();

            // Huge debug...
            if (this.getLogger().isLoggable(Level.FINE)) {
                if (UtilFactory.getWSDLUtil().isDescriptionContaining(
                        this.component.getNativeWsdl().getDescription(), exchange.getEndpoint())) {
                    this.getLogger().log(Level.FINE, "exchange to process internally received");
                } else {
                    this.getLogger().log(Level.FINE,
                            "exchange to process with the SU manager received");
                }
                
                this.getLogger().fine(
                        "MexId : " + exchange.getExchangeId()
                                + " : Got a notification with operation = '" + opName + "'");
                try {
                    this.getLogger().fine(
                            "MexId : " + exchange.getExchangeId() + " : "
                                    + XMLUtil.createStringFromDOMDocument(docContent));
                } catch (TransformerException e) {
                    getLogger().log(Level.WARNING,
                            "Can not display message content due to transformer exception");
                }
            }
            // ============================================================
            // ========= NotificationConsumer interface operation =========
            // ============================================================
            if (WsnbConstants.NOTIFY_QNAME.getLocalPart().equalsIgnoreCase(opName)) {
                if (exchange.isInOnlyPattern() || exchange.isRobustInOnlyPattern()
                        || exchange.isInOptionalOutPattern()) {
                    // this.wsnbrService.notify(docContent);
                    wsnbrService.notify(wsnReader.readNotify(docContent));
                } else {
                    // error exchange
                    exchange
                    .setError(new Exception("Uncompatible MEP '"
                            + exchange.getPattern().toString() + "' and operation '"
                            + opName + "'"));
                }
            }
            // =======================================================================
            // ============ RegisterPublisher{Manager} interfaces operations
            // =========
            // =======================================================================
            else if (WsnbrConstants.REGISTER_PUBLISHER_QNAME.getLocalPart()
                    .equalsIgnoreCase(opName)) {
                if (exchange.isInOutPattern() || exchange.isInOptionalOutPattern()) {
                    try {
                        outWsnContent = wsnWriter.writeRegisterPublisherResponse(wsnbrService
                                .registerPublisher(wsnReader.readRegisterPublisher(docContent)));
                        exchange.setOutMessageContent(outWsnContent);
                    } catch (final WSNotificationFault e) {
                        // fault exchange
                        fault = exchange.createFault();
                        fault.setContent(UtilFactory.getSourceUtil().createDOMSource(
                                e.getDocumentFragment()));
                        exchange.setFault(fault);
                    }
                } else {
                    // error exchange
                    exchange
                    .setError(new Exception("Uncompatible MEP '"
                            + exchange.getPattern().toString() + "' and operation '"
                            + opName + "'"));
                }
            } else if (WsnbrConstants.DESTROY_REGISTRATION_QNAME.getLocalPart()
                    .equalsIgnoreCase(opName)) {
                if (exchange.isInOutPattern() || exchange.isInOptionalOutPattern()) {
                    try {
                        outWsnContent = wsnWriter
                        .writeDestroyRegistrationResponse(wsnbrService
                                .destroyRegistration(wsnReader
                                        .readDestroyRegistration(docContent)));
                        exchange.setOutMessageContent(outWsnContent);
                    } catch (final WSNotificationFault e) {
                        // fault exchange
                        fault = exchange.createFault();
                        fault.setContent(UtilFactory.getSourceUtil().createDOMSource(
                                e.getDocumentFragment()));
                        exchange.setFault(fault);
                    }
                } else {
                    // error exchange
                    exchange
                    .setError(new Exception("Uncompatible MEP '"
                            + exchange.getPattern().toString() + "' and operation '"
                            + opName + "'"));
                }
            }
            // =============================================================
            // ========= NotificationProducer interface operations =========
            // =============================================================
            else if (WsnbConstants.UNSUBSCRIBE_QNAME.getLocalPart().equalsIgnoreCase(opName)) {
                if (exchange.isInOutPattern() || exchange.isInOptionalOutPattern()) {

                    try {
                        outWsnContent = wsnWriter.writeUnsubscribeResponse(wsnbrService
                                .unsubscribe(wsnReader.readUnsubscribe(docContent)));
                        exchange.setOutMessageContent(outWsnContent);
                    } catch (final WSNotificationFault e) {
                        // fault exchange
                        fault = exchange.createFault();
                        fault.setContent(UtilFactory.getSourceUtil().createDOMSource(
                                e.getDocumentFragment()));
                        exchange.setFault(fault);
                    }
                } else {
                    // error exchange
                    exchange
                    .setError(new Exception("Uncompatible MEP '"
                            + exchange.getPattern().toString() + "' and operation '"
                            + opName + "'"));
                }
            } else if (WsnbConstants.SUBSCRIBE_QNAME.getLocalPart().equalsIgnoreCase(opName)) {
                if (exchange.isInOutPattern() || exchange.isInOptionalOutPattern()) {
                    try {
                        outWsnContent = wsnWriter.writeSubscribeResponse(wsnbrService
                                .subscribe(wsnReader.readSubscribe(docContent)));
                        exchange.setOutMessageContent(outWsnContent);
                    } catch (final WSNotificationFault e) {
                        // fault exchange
                        fault = exchange.createFault();
                        fault.setContent(UtilFactory.getSourceUtil().createDOMSource(
                                e.getDocumentFragment()));
                        exchange.setFault(fault);
                    }
                } else {
                    // error exchange
                    exchange
                    .setError(new Exception("Uncompatible MEP '"
                            + exchange.getPattern().toString() + "' and operation '"
                            + opName + "'"));
                }
            }

            // ==================================================
            // ======= Others Operations and unknown ones =======
            // ==================================================

            else if (this.GET_SUPPORTED_TOPICS_QNAME.getLocalPart().equalsIgnoreCase(opName)) {
                if (exchange.isInOutPattern() || exchange.isInOptionalOutPattern()) {
                    /*try {*/
                    outWsnContent = wsnbrService.getSupportedTopicsSet();
                    exchange.setOutMessageContent(outWsnContent);
                    /*} catch (final WSNotificationFault e) {
                        // fault exchange
                        fault = exchange.createFault();
                        fault.setContent(UtilFactory.getSourceUtil().createDOMSource(
                                e.getDocumentFragment()));
                        exchange.setFault(fault);
                    }*/
                } else {
                    // error exchange
                    exchange
                    .setError(new Exception("Uncompatible MEP '"
                            + exchange.getPattern().toString() + "' and operation '"
                            + opName + "'"));
                }
            }

            // == Unknown operations ==
            
            else {
                // error exchange
                exchange.setError(new Exception(
                        "Unknown - or noy yet implemented - operation's name '" + opName + "'"));
            }
        } catch (final WSNotificationNotImplementedException e) {
            this.getLogger().warning(
                    "WS-NOTIFICATION FEATURE NOT IMPLEMENTED EXCEPTION : " + e.getMessage());
            exchange.setError(new Exception(e));
        } catch (final WSNotificationException e) {
            this.getLogger().warning(
                    "An error occurs during the process of WS-notification operation : "
                    + e.getMessage());
            exchange.setError(new Exception(e));
        } catch (final MessagingException e) {
            this.getLogger().warning(
                    "An error occurs during the treatment of an exchange : " + e.getMessage());
            exchange.setError(new Exception(e));
        } catch (final WSDLException e) {
            this.getLogger().warning(
                    "An error occurs during the treatment of an exchange : " + e.getMessage());
            exchange.setError(new Exception(e));
        }
        return true;
    }
}
